/*
*   Floating point routines for Commodore plus/4
*   Copyright (C) 2004 Ingo D. Rullhusen <d01c@uni-bremen.de>
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation; either version 2 of the License, or
*   (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
*/


#include <stdlib.h>
#include <stdio.h>

#include <peekpoke.h>

#include "cbm_math.h"


extern void ROM_jsr( unsigned addr );

extern void ROM_jsr_ator();


static const char CBMr_PI[] = { 0x82, 0xc9, 0x0f, 0xda, 0xa2, 0x00 };
const cbm_real *cbm_PI = (cbm_real*)&CBMr_PI;

static const char CBMr_LN10[] = { 0x82, 0x93, 0x5d, 0x8d, 0xe2, 0x00 };
const cbm_real *cbm_LN10 = (cbm_real*)&CBMr_LN10;

static const char CBMr_0[] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
const cbm_real *cbm_0 = (cbm_real*)&CBMr_0;

static const char CBMr_1[] = { 0x81, 0x80, 0x00, 0x00, 0x00, 0x00 };
const cbm_real *cbm_1 = (cbm_real*)&CBMr_1;

static const char CBMr_10[] = { 0x84, 0xa0, 0x00, 0x00, 0x00, 0x00 };
const cbm_real *cbm_10 = (cbm_real*)&CBMr_10;


static fac2out( cbm_real *out )
{
   ROM_jsr( 0xa2a0 );
   POKEW(  (unsigned int)out,    PEEKW( 0x0061 ) );
   POKEW( ((unsigned int)out)+2, PEEKW( 0x0063 ) );
   POKEW( ((unsigned int)out)+4, PEEKW( 0x0065 ) );
}

static in2fac( const cbm_real *in )
{
   POKEW( 0x0061, PEEKW(  (unsigned int)in    ) );
   POKEW( 0x0063, PEEKW( ((unsigned int)in)+2 ) );
   POKEW( 0x0065, PEEKW( ((unsigned int)in)+4 ) );
   POKEW( 0x0067, 0x0000 );   // poly carry fac

   if ( (PEEK(0x0066) ^ PEEK(0x006e)) & 0x80 )
      POKE( 0x6f, 0xff );  // sign neq
   else
      POKE( 0x6f, 0x00 );  // sign eq

   POKE( 0x0070, 0x00 );      // round fac
}

static in2argfac( const cbm_real *in1, const cbm_real *in2 )
{
   POKEW( 0x0069, PEEKW(  (unsigned int)in1    ) );
   POKEW( 0x006b, PEEKW( ((unsigned int)in1)+2 ) );
   POKEW( 0x006d, PEEKW( ((unsigned int)in1)+4 ) );

   POKEW( 0x0061, PEEKW(  (unsigned int)in2    ) );
   POKEW( 0x0063, PEEKW( ((unsigned int)in2)+2 ) );
   POKEW( 0x0065, PEEKW( ((unsigned int)in2)+4 ) );

   POKE( 0x67, 0x00 );     // poly
   POKE( 0x68, 0x00 );     // carry fac
   if ( (in1->S ^ in2->S) & 0x80 )
      POKE( 0x6f, 0xff );  // sign neq
   else
      POKE( 0x6f, 0x00 );  // sign eq

   POKE( 0x70, 0x00 );     // round fac 
}

static in2arg( const cbm_real *in )
{
   POKEW( 0x0069, PEEKW(  (unsigned int)in    ) );
   POKEW( 0x006b, PEEKW( ((unsigned int)in)+2 ) );
   POKEW( 0x006d, PEEKW( ((unsigned int)in)+4 ) );

   if ( (PEEK(0x0066) ^ PEEK(0x006e)) & 0x80 )
      POKE( 0x6f, 0xff );  // sign neq
   else
      POKE( 0x6f, 0x00 );  // sign eq
}


void cbm_rtor( cbm_real *out, const cbm_real *in )
{
   POKEW(  (unsigned int)out,    PEEKW(  (unsigned int)in    ) );
   POKEW( ((unsigned int)out)+2, PEEKW( ((unsigned int)in)+2 ) );
   POKEW( ((unsigned int)out)+4, PEEKW( ((unsigned int)in)+4 ) );
}


void cbm_ator( cbm_real *out, const char *nptr, char **endptr )
{
   char *Eptr;
   unsigned int adr_t;

   adr_t = PEEKW( 0x003b );
   POKEW( 0x003b, (unsigned int)nptr );

   ROM_jsr_ator();

   if ( *( Eptr = (char*)PEEKW( 0x003b ) ) == 'E' )
   {
      *Eptr = 'e';
      POKEW( 0x003b, (unsigned int)nptr );
      ROM_jsr_ator();
      *Eptr = 'E';
   }

   if ( endptr != NULL ) *endptr = (char*)PEEKW( 0x003b );

   fac2out(out);

   POKEW( 0x003b, adr_t );
}

void cbm_rtoa( char *out, int size, const cbm_real *in )
{
   int i;

   in2fac(in);

   ROM_jsr( 0xa46f );

   i = 0;
   while ( ( out[i]=PEEK(0x0100+i) ) != '\0' )
   {
      ++i;
      if ( i >= size )
      {
         for ( i=0; i<size-1; i++ ) out[i] = '*';
         out[size-1] = '\0';
         break;
      }
   }
}


void cbm_itor( cbm_real *out, const int *in )
{
   union { int i;
           unsigned char b[2]; } in_t;
   in_t.i  = *in;

   POKE(  (unsigned int)out,     0x90 );
   POKEW( ((unsigned int)out)+3, 0x0000 );

   if ( in_t.i < 0 )
   {
      in_t.i ^= 0xffff;
      ++in_t.i;
      POKE(  ((unsigned int)out)+5, 0xff );
   }
   else
   {
      POKE(  ((unsigned int)out)+5, 0x00 );
   }

   while ( !( in_t.i & 0x8000 ) && ((unsigned char)*out) > 0x81 )
   {
      in_t.i <<= 1;
      --((char)*out);
   }

   POKE(  ((unsigned int)out)+1, in_t.b[1] );
   POKE(  ((unsigned int)out)+2, in_t.b[0] );
}

void cbm_rtoi( int *out, const cbm_real *in )
{
   in2fac(in);

   ROM_jsr( 0xa327 );

   POKE( ((unsigned int)out),   PEEK( 0x65 ) );
   POKE( ((unsigned int)out)+1, PEEK( 0x64 ) );
}


int cbm_signum( const cbm_real *in )
{
   if ( PEEK( ((unsigned int)in)+5 ) & 0x80 )
      return(-1);
   else if ( *in == 0x00 ) return(0);
   return(+1);
}

int cbm_compare( const cbm_real *op1, const cbm_real *op2 )
{
   in2argfac(op1,op2);

   ROM_jsr( 0x9e87 );

   if ( PEEK( 0x0066 ) & 0x80 )
      return(-1);
   else if ( PEEK( 0x0061 ) == 0x00 ) return(0);
   return(+1);
}


void cbm_add( cbm_real *out, const cbm_real *in1, const cbm_real *in2 )
{
   in2argfac(in1,in2);

   ROM_jsr( 0x9e9e );

   fac2out(out);
}

void cbm_sub( cbm_real *out, const cbm_real *in1, const cbm_real *in2 )
{
   in2argfac(in1,in2);

   ROM_jsr( 0x9e87 );

   fac2out(out);
}

void cbm_mult( cbm_real *out, const cbm_real *in1, const cbm_real *in2 )
{
   in2argfac(in1,in2);

   ROM_jsr( 0xa07b );

   fac2out(out);
}

void cbm_divide( cbm_real *out, const cbm_real *in1, const cbm_real *in2 )
{
   in2argfac(in1,in2);

   ROM_jsr( 0xa197 );

   fac2out(out);
}

void cbm_pow( cbm_real *out, const cbm_real *in1, const cbm_real *in2 )
{
   in2argfac(in1,in2);

   ROM_jsr( 0xa5ee );

   fac2out(out);
}


void cbm_fabs( cbm_real *out, const cbm_real *in )
{
   POKEW(  (unsigned int)out,    PEEKW(  (unsigned int)in    ) );
   POKEW( ((unsigned int)out)+2, PEEKW( ((unsigned int)in)+2 ) );
   POKE(  ((unsigned int)out)+4, PEEK(  ((unsigned int)in)+4 ) );
   POKE(  ((unsigned int)out)+5, PEEK(  ((unsigned int)in)+5 ) & 0x7f );
}


void cbm_sqrt( cbm_real *out, const cbm_real *in )
{
   in2fac(in);

   ROM_jsr( 0xa5e4 );
   
   fac2out(out);
}


void cbm_exp( cbm_real *out, const cbm_real *in )
{
   in2fac(in);

   ROM_jsr( 0xa660 );

   fac2out(out);
}


void cbm_log( cbm_real *out, const cbm_real *in )
{
   in2fac(in);
   
   ROM_jsr( 0xa01e );
      
   fac2out(out);
}


void cbm_cos( cbm_real *out, const cbm_real *in )
{
   in2fac(in);
   
   ROM_jsr( 0xaa70 );

   fac2out(out);
}


void cbm_sin( cbm_real *out, const cbm_real *in )
{
   in2fac(in);
   
   ROM_jsr( 0xaa77 );

   fac2out(out);
}


void cbm_tan( cbm_real *out, const cbm_real *in )
{
   in2fac(in);
   
   ROM_jsr( 0xaac0 );

   fac2out(out);
}


void cbm_atan( cbm_real *out, const cbm_real *in )
{
   in2fac(in);
   
   ROM_jsr( 0xab1a );

   fac2out(out);
}


void cbm_log10( cbm_real *out, const cbm_real *in )
{
   in2fac(in);
   
   ROM_jsr( 0xa01e );
   ROM_jsr( 0xa291 );

   in2fac(cbm_LN10);

   ROM_jsr( 0xa197 );

   fac2out(out);
}


void cbm_asin( cbm_real *out, const cbm_real *in )
{
   in2argfac(in,in);
   ROM_jsr( 0xa07b );
   in2arg(cbm_1);
   ROM_jsr( 0x9e87 );
   ROM_jsr( 0xa5e4 );

   in2arg(in);
   ROM_jsr( 0xa197 );
   ROM_jsr( 0xab1a );

   fac2out(out);
}


void cbm_acos( cbm_real *out, const cbm_real *in )
{
   in2argfac(in,in);
   ROM_jsr( 0xa07b );
   in2arg(cbm_1);
   ROM_jsr( 0x9e87 );
   ROM_jsr( 0xa5e4 );

   in2arg(in);
   ROM_jsr( 0xa197 );
   ROM_jsr( 0xab1a );

   in2arg(cbm_PI);
   __asm__(" dec $69 ");

   ROM_jsr( 0x9e87 );

   fac2out(out);
}


void cbm_sinh( cbm_real *out, const cbm_real *in )
{
   in2fac(in);
   ROM_jsr( 0xa660 );
   fac2out(out);

   in2fac(in);
   __asm__(" lda #$80 \n"
           " eor $66  \n"
           " sta $66  \n");
   ROM_jsr( 0xa660 );

   in2arg(out);
   ROM_jsr( 0x9e87 );

   __asm__(" dec $61 ");

   fac2out(out);
}


void cbm_cosh( cbm_real *out, const cbm_real *in )
{
   in2fac(in);
   ROM_jsr( 0xa660 );
   fac2out(out);

   in2fac(in);
   __asm__(" lda #$80 \n"
           " eor $66  \n"
           " sta $66  \n");
   ROM_jsr( 0xa660 );

   in2arg(out);
   ROM_jsr( 0x9e9e );

   __asm__(" dec $61 ");

   fac2out(out);
}


void cbm_tanh( cbm_real *out, const cbm_real *in )
{
   cbm_real ep, em;

   in2fac(in);
   if ( PEEK( 0x0061 ) > 0x86 )
   {
      cbm_rtor(out,cbm_1);
      if ( PEEK( 0x0066 ) & 0x80 ) POKE( ((unsigned int)out)+5, 0xff );
      return;
   }

   ROM_jsr( 0xa660 );
   fac2out(&ep);

   in2fac(in);
   __asm__(" lda #$80 \n"
           " eor $66  \n"
           " sta $66  \n");
   ROM_jsr( 0xa660 );
   fac2out(&em);

   in2arg(&ep);
   ROM_jsr( 0x9e87 );
   fac2out(out);

   in2argfac(&ep,&em);
   ROM_jsr( 0x9e9e );

   in2arg(out);
   ROM_jsr( 0xa197 );

   fac2out(out);
}

void cbm_asinh( cbm_real *out, const cbm_real *in )
{
   in2argfac(in,in);
   ROM_jsr( 0xa07b );
   
   ROM_jsr( 0xa291 );
   in2fac(cbm_1);
   ROM_jsr( 0x9e9e );
   ROM_jsr( 0xa5e4 );

   in2arg(in);
   POKE( 0x006e, 0x00 );
   POKE( 0x006f, 0x00 );
   ROM_jsr( 0x9e9e );

   ROM_jsr( 0xa01e );
   if ( PEEK( ((unsigned int)in)+5 ) & 0x80 ) POKE( 0x0066, 0xff );

   fac2out(out);
}

void cbm_acosh( cbm_real *out, const cbm_real *in )
{
   in2argfac(in,in);
   ROM_jsr( 0xa07b );
   
   ROM_jsr( 0xa291 );
   in2fac(cbm_1);
   ROM_jsr( 0x9e87 );
   ROM_jsr( 0xa5e4 );

   in2arg(in);
   ROM_jsr( 0x9e9e );

   ROM_jsr( 0xa01e );

   fac2out(out);
}

void cbm_atanh( cbm_real *out, const cbm_real *in )
{
   in2argfac(cbm_1,in);
   ROM_jsr( 0x9e9e );
   fac2out(out);

   in2argfac(cbm_1,in);
   ROM_jsr( 0x9e87 );

   in2arg(out);
   ROM_jsr( 0xa197 );

   ROM_jsr( 0xa01e );
   __asm__(" dec $61 ");

   fac2out(out);
}
