/*
*   Floating point routines for Commodore plus/4
*   Copyright (C) 2004 Ingo D. Rullhusen <d01c@uni-bremen.de>
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation; either version 2 of the License, or
*   (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
*/


#ifdef __CC65__

#ifndef __PLUS4__
#ifndef __C16__
#error cbm_math works with C16 C116 plus/4 only!
#endif
#endif

typedef struct
{
   unsigned char EX, M1, M2, M3, M4, S;
} cbm_real;

#else

typedef double cbm_real;

#endif


extern const cbm_real *cbm_PI;
extern const cbm_real *cbm_LN10;
extern const cbm_real *cbm_0;
extern const cbm_real *cbm_1;
extern const cbm_real *cbm_10;


void cbm_rtor( cbm_real *out, const cbm_real *in );

void cbm_ator( cbm_real *out, const char *in, char **endptr );

void cbm_rtoa( char *out, int size, const cbm_real *in );

void cbm_itor( cbm_real *out, const int *in );

void cbm_rtoi( int *out, const cbm_real *in );


int cbm_signum( const cbm_real *in );

int cbm_compare( const cbm_real *op1, const cbm_real *op2 );


void cbm_add( cbm_real *out, const cbm_real *in1, const cbm_real *in2 );

void cbm_sub( cbm_real *out, const cbm_real *in1, const cbm_real *in2 );

void cbm_mult( cbm_real *out, const cbm_real *in1, const cbm_real *in2 );

void cbm_divide( cbm_real *out, const cbm_real *in1, const cbm_real *in2 );

void cbm_pow( cbm_real *out, const cbm_real *in1, const cbm_real *in2 );


void cbm_fabs( cbm_real *out, const cbm_real *in );

void cbm_sqrt( cbm_real *out, const cbm_real *in );

void cbm_exp( cbm_real *out, const cbm_real *in );

void cbm_log( cbm_real *out, const cbm_real *in );

void cbm_log10( cbm_real *out, const cbm_real *in );

void cbm_cos( cbm_real *out, const cbm_real *in );

void cbm_sin( cbm_real *out, const cbm_real *in );

void cbm_tan( cbm_real *out, const cbm_real *in );

void cbm_atan( cbm_real *out, const cbm_real *in );

void cbm_asin( cbm_real *out, const cbm_real *in );

void cbm_acos( cbm_real *out, const cbm_real *in );

void cbm_sinh( cbm_real *out, const cbm_real *in );

void cbm_cosh( cbm_real *out, const cbm_real *in );

void cbm_tanh( cbm_real *out, const cbm_real *in );

void cbm_asinh( cbm_real *out, const cbm_real *in );

void cbm_acosh( cbm_real *out, const cbm_real *in );

void cbm_atanh( cbm_real *out, const cbm_real *in );
