/*
*   Floating point routines for Commodore plus/4
*   Copyright (C) 2004 Ingo D. Rullhusen <d01c@uni-bremen.de>
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation; either version 2 of the License, or
*   (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
*/


#include <stdlib.h>
#include <stdio.h>

#include <math.h>

#include "cbm_math.h"


static cbm_real CBMr_PI = M_PI;
const cbm_real *cbm_PI = &CBMr_PI;

static cbm_real CBMr_LN10 = M_LN10;
const cbm_real *cbm_LN10 = &CBMr_LN10;

static cbm_real CBMr_0 = 0.0;
const cbm_real *cbm_0 = &CBMr_0;

static cbm_real CBMr_1 = 1.0;
const cbm_real *cbm_1 = &CBMr_1;

static cbm_real CBMr_10 = 10.0;
const cbm_real *cbm_10 = &CBMr_10;


void cbm_rtor( cbm_real *out, const cbm_real *in ) { *out = *in; }

void cbm_ator( cbm_real *out, const char *nptr, char **endptr )
{
   *out = strtod( nptr, endptr );
   if ( endptr ) while( **endptr == ' ' ) (*endptr)++;
 }

void cbm_rtoa( char *out, int size, const cbm_real *in )
{
   int i;

   if ( snprintf(out,size,"% .9g",*in) >= size )
   {
      for ( i=0; i<size-1; i++ ) out[i] = '*';
      out[size-1] = '\0';
   }
}

void cbm_itor( cbm_real *out, const int *in ) { *out = *in; }

void cbm_rtoi( int *out, const cbm_real *in ) { *out = (int)*in; }


int cbm_signum( const cbm_real *in )
{
   if ( *in < 0.0 ) return(-1);
   if ( *in > 0.0 ) return(+1);
   return(0);
}

int cbm_compare( const cbm_real *op1, const cbm_real *op2 )
{
   if ( *op1 < *op2 ) return(-1);
   if ( *op1 > *op2 ) return(+1);
   return(0);
}


void cbm_add( cbm_real *out, const cbm_real *in1, const cbm_real *in2 )
{ *out = *in1 + *in2; }

void cbm_sub( cbm_real *out, const cbm_real *in1, const cbm_real *in2 )
{ *out = *in1 - *in2; }

void cbm_mult( cbm_real *out, const cbm_real *in1, const cbm_real *in2 )
{ *out = *in1 * *in2; }

void cbm_divide( cbm_real *out, const cbm_real *in1, const cbm_real *in2 )
{ *out = *in1 / *in2; }

void cbm_pow( cbm_real *out, const cbm_real *in1, const cbm_real *in2 )
{ *out = pow( *in1, *in2 ); }

void cbm_fabs( cbm_real *out, const cbm_real *in ) { *out = fabs(*in); }

void cbm_sqrt( cbm_real *out, const cbm_real *in ) { *out = sqrt(*in); }

void cbm_exp( cbm_real *out, const cbm_real *in ) { *out = exp(*in); }

void cbm_log( cbm_real *out, const cbm_real *in ) { *out = log(*in); }

void cbm_log10( cbm_real *out, const cbm_real *in ) { *out = log10(*in); }

void cbm_cos( cbm_real *out, const cbm_real *in ) { *out = cos(*in); }

void cbm_sin( cbm_real *out, const cbm_real *in ) { *out = sin(*in); }

void cbm_tan( cbm_real *out, const cbm_real *in ) { *out = tan(*in); }

void cbm_atan( cbm_real *out, const cbm_real *in ) { *out = atan(*in); }

void cbm_asin( cbm_real *out, const cbm_real *in ) { *out = asin(*in); }

void cbm_acos( cbm_real *out, const cbm_real *in ) { *out = acos(*in); }

void cbm_sinh( cbm_real *out, const cbm_real *in ) { *out = sinh(*in); }

void cbm_cosh( cbm_real *out, const cbm_real *in ) { *out = cosh(*in); }

void cbm_tanh( cbm_real *out, const cbm_real *in ) { *out = tanh(*in); }

void cbm_asinh( cbm_real *out, const cbm_real *in ) { *out = asinh(*in); }

void cbm_acosh( cbm_real *out, const cbm_real *in ) { *out = acosh(*in); }

void cbm_atanh( cbm_real *out, const cbm_real *in ) { *out = atanh(*in); }
